!********************************************************************** 
module constants 
      implicit none 
      double precision,  parameter  ::  pi = 3.1415926535897932d0
      ! strong coupling constant -- when used with N_C
      double precision,  parameter  ::  alpha_s = 0.177777777777777777778d0
      ! Number of colours
      double precision,  parameter  ::  nc = 3
      ! riemann zeta(3)
      double precision,  parameter  ::  zeta3 = 1.202d0
      ! Allow different value for alpha_s for cases where it is not
      ! combined with N_c (number of colours) -- e.g. dipole-dipole
      ! interaction
      double precision,  parameter  ::  alpha_no_nc = alpha_s
end module constants
    
!********************************************************************** 
module cuts 
      double precision r_cut_lo, r_cut_hi ! upper and lower cutoffs
      logical hi_cut_implem ! should be set .true. if r_cut_hi > r_cut_lo
      ! Temporary: for statistics
      integer iangrej, iangtot   
      integer ibrej, ibtot
      data iangrej, iangtot, ibrej, ibtot /4*0/
end module cuts
                                                                        
!----------------------------------------------------------------------
! These variables describe positions in  arrays which are accessed from a
! wide variety of places. Therefore it is useful to keep them global to
! ensure consistent positioning of elements within that array.
! 
! GPS 7/11/95
!----------------------------------------------------------------------
module evoln_intrfc
      ! gluon_n describes position in gluon_set array (only present when
      ! fixed arrays are used to store all dipoles and gluons)
      ! dipole_n describes position in dipole_set array -- used to hold
      ! all dipoles when fixed arrays are the storage, or just the initial
      ! dipole of each onium when dynamicallyallocated storage is used.
      integer gluon_n, dipole_n
end module evoln_intrfc

                                                                        
!********************************************************************** 


!**********************************************************************
module iomodule
      ! The standard input and output devices
      integer, parameter   ::  iprm      = 10  ! parameter files
      integer, parameter   ::  idat      = 12  ! formatted data
      integer, parameter   ::  iufdt     = 14  ! unformatted data

      ! Devices for use with paw (and other graph progs)
      ! (to ensure that devices don't conflict)
      integer, parameter   ::  ipaw_kmc  = 80
      integer, parameter   ::  ipaw_dat1 = 81
      integer, parameter   ::  ipaw_dat2 = 82
      ! Device for reading startup file 
      integer, parameter   ::  istartup  = 20
      !------------------------------------------------------------------
      ! The section between INSTALL_START and INSTALL_END is generated
      ! automatically by the installation script
      ! DO NOT ALTER THE NEXT LINE: USED BY INSTALLATION SCRIPT
      !INSTALL_START
      character(len=*), parameter ::&
       & top_dir='/home/salam/work/oedipus'
      !INSTALL_END
      ! DO NOT ALTER THE PREVIOUS LINE: USED BY INSTALLATION SCRIPT
      !------------------------------------------------------------------
      ! String containing directory for startup files
      character(len=*), parameter :: start_dir = top_dir//'/starters/'
      ! String containing shell script which makes backup copies of the
      ! data and parameter files -- space at end is important
      character(len=*), parameter :: backup_cmd = top_dir//'/bin/do_backup '
end module iomodule
!**********************************************************************




!DEBUG -- should hold number evolutions done so far -- allows any problems
! with evolution to be traced more easily -- it is the user's
! responsibility to set this.
module emergency
      integer n_dip_emergency
end module emergency
!----------------------------------------------------------------------
subroutine emergency_stop
      use emergency
      write(6,*) "Emergency Stop"
      write(6,*) "Number of evolved dipoles is", n_dip_emergency
!      stop
end subroutine emergency_stop
