!----------------------------------------------------------------------
! A variety of routines that need to be implementation specific
!----------------------------------------------------------------------

!----------------------------------------------------------------------
! Reading and writing very large 2d arrays (with unformatted i/o) causes
! problems on certain systems -- so provide routines which split up
! the reading and writing if that is necessary.
! Splitting is required for nag
!----------------------------------------------------------------------
subroutine lcl_write_2d(idev, array)
      implicit none
      integer, intent(in) :: idev
      real,    intent(in) :: array(1:, 1:)
      !------------------------------------------------------------
      integer  ulim(2), i

      ulim = ubound(array)
      do i = 1, ulim(2)
         write(idev) array(:,i)
      end do
      
end subroutine lcl_write_2d


subroutine lcl_read_2d(idev, array)
      implicit none
      integer, intent(in)  :: idev
      real,    intent(out) :: array(1:, 1:)
      !------------------------------------------------------------
      integer  ulim(2), i

      ulim = ubound(array)
      do i = 1, ulim(2)
         read(idev) array(:,i)
      end do

end subroutine lcl_read_2d


!----------------------------------------------------------------------
! Interfaces to the nag f90 iargc and getarg routines. Must use the nag
! f90 module f90_unix
!
! GPS 4/11/95 (CCN8 9)
!----------------------------------------------------------------------
integer function lcl_iargc()
      use f90_unix
      implicit none

      lcl_iargc = iargc()
end function lcl_iargc

subroutine lcl_getarg(k, argk)
      use f90_unix
      implicit none
      integer,      intent(in)  :: k
      character(*), intent(out) :: argk

      call getarg(k, argk)
end subroutine lcl_getarg

!-- nag f90 does not seem to have a 'system' routine, so just output an
! error message and continue execution.
subroutine lcl_system(string)
      implicit none
      character(*), intent(in) :: string
      !------------------------------------------------------------
      !integer return_val, system

      write(0,*) 'The system call "system" does not seem to be implemented'
      write(0,*) 'yet with the NAG compiler'
      write(0,*) 'If this routine was called to do backups, then backups'
      write(0,*) 'will not be performed'
end subroutine lcl_system
