!----------------------------------------------------------------------
! A variety of routines that need to be implementation specific
! This set is written so as to use only standard f90.
!----------------------------------------------------------------------

!----------------------------------------------------------------------
! Reading and writing very large 2d arrays (with unformatted i/o) causes
! problems on certain systems -- so provide routines which split up
! the reading and writing if that is necessary.
! Splitting is safest way of doing things.
!----------------------------------------------------------------------
subroutine lcl_write_2d(idev, array)
      implicit none
      integer, intent(in) :: idev
      real,    intent(in) :: array(1:, 1:)
      !------------------------------------------------------------
      integer  ulim(2), i

      ulim = ubound(array)
      do i = 1, ulim(2)
         write(idev) array(:,i)
      end do
      
end subroutine lcl_write_2d


subroutine lcl_read_2d(idev, array)
      implicit none
      integer, intent(in)  :: idev
      real,    intent(out) :: array(1:, 1:)
      !------------------------------------------------------------
      integer  ulim(2), i

      ulim = ubound(array)
      do i = 1, ulim(2)
         read(idev) array(:,i)
      end do

end subroutine lcl_read_2d


!----------------------------------------------------------------------
! Routines to provide equivalent of the unix f77 iargc and getarg
! commands. Because they were non-standard, they need to be implemented in
! a different way: the first time the program calls iargc_std or
! getarg_std the program queries the user for the arguments (each argument
! on a separate line), using an & to indicate the end of the arguments. 
!
! GPS 4/1/96
!----------------------------------------------------------------------
module f90_unix_std
      implicit none
      !-- define what is accessible outside this module -----------
      private
      public iargc_std, getarg_std
      !------------------------------------------------------------
      logical  :: args_input = .false.
      integer  :: narg
      type arg
         character(80)      :: string
         type(arg), pointer :: next
      end type arg
      type(arg),    target  :: first_arg
      
      contains

      !-- prompts user for each argument
      subroutine inputs_args
            implicit none
            character*80       :: string
            type(arg), pointer :: current_arg

            write(6,'(a)') "Please input the command line arguments below"
            write(6,'(a)') "(enter & followed by return to finish)"
            narg = 0
            current_arg => first_arg
loop:       do 
               write(6,'(a,i2,a)', advance= 'no') "Argument ",narg+1,": "
               read(5,*) string
               if (string == "&") exit loop
               narg = narg + 1
               current_arg%string = string 
               allocate(current_arg%next)
               current_arg => current_arg%next
            end do loop
            nullify(current_arg%next)
            args_input = .true.
      end subroutine inputs_args
      
      !-- returns number of arguments
      integer function iargc_std()
            implicit none
            
            if (.not. args_input) call inputs_args
            iargc_std = narg
      end function iargc_std
      
      !-- returns string with trailing blanks removed
      subroutine getarg_std(k, argk)
            implicit none
            integer,      intent(in)  :: k
            character(*), intent(out) :: argk
            type(arg),    pointer     :: current_arg
            !--------------------------------------------------
            integer i

            if (.not. args_input) call inputs_args
            if (k > narg) then
               write(0,'(a,i2)')   "Error: program requested argument ",k
               write(0,'(a,i2,a)') "but only ",k," arguments are present"
               write(0,'(a)')      "STOP"
               stop
            end if
            
            !-- would normally be the program name, but this is not
            ! accessible here
            if (k == 0) then
               argk = "Program_name"
               return
            end if
            

            current_arg => first_arg
            do i = 1, k-1
               current_arg => current_arg%next
            end do
            i = len_trim(current_arg%string)
            !-- will this work?
            argk = current_arg%string(1:i)
      end subroutine getarg_std
end module f90_unix_std


integer function lcl_iargc()
      use f90_unix_std
      implicit none

      lcl_iargc = iargc_std()
end function lcl_iargc


subroutine lcl_getarg(k, argk)
      use f90_unix_std
      implicit none
      integer,      intent(in)  :: k
      character(*), intent(out) :: argk

      call getarg_std(k, argk)
end subroutine lcl_getarg


subroutine lcl_system(string)
      implicit none
      character(*), intent(in) :: string
      !------------------------------------------------------------
      !integer return_val, system

      write(0,'(a)') 'Warning: the system call "system", is not part of the'
      write(0,'(a)') 'Fortran 90 standard and not available in this'
      write(0,'(a)') "operating system's implementation of f90. If this"
      write(0,'(a)') 'routine was called to perform backups then the'
      write(0,'(a)') 'backups will not be made'
end subroutine lcl_system
