module sub_defs 
      !===================================================================
      ! Routines related to initialising onia, evolving them and then
      ! extracting the resulting dipoles
      interface
         subroutine extract_onium_dpls(onium, y, extracted_dipoles, n)
               use gluons 
               implicit none 
               type(dipole),     intent(in), target        :: onium 
               double precision, intent(in)                :: y 
               type(dpl_pntr), dimension(:), intent(inout) ::&
                    & extracted_dipoles 
               integer,        intent(inout)               :: n 
         end subroutine extract_onium_dpls
      end interface
      
      interface 
         recursive subroutine extract_dipoles(top,y,extracted_dipoles,n) 
               use gluons 
               implicit none 
               type(dipole),   intent(in), target :: top 
               double precision, intent(in)       :: y 
               type(dpl_pntr), intent(inout)      :: extracted_dipoles(:)
               integer,        intent(inout)      :: n 
         end subroutine extract_dipoles
      end interface

      interface
         subroutine init_onium(onium, onm_sz, phi) 
               use gluons 
               implicit none 
               type(dipole), pointer         :: onium
               double precision,  intent(in) :: onm_sz, phi
         end subroutine init_onium
      end interface
      

      interface 
         subroutine init_onium_rnd_phi(onium, onm_size)
               use gluons
               implicit none
               type(dipole),     pointer     :: onium
               double precision, intent(in)  :: onm_size
         end subroutine init_onium_rnd_phi
      end interface

      !============================================================
      ! Onium Onium interactions
      interface
         subroutine onm_onm_grid(ext_dips1, n1, ext_dips2, n2, grid, gh_lo&
              & , gh_int, gv_lo, gv_int) 
               use gluons
               implicit none
               type(dpl_pntr), dimension(:), intent(in)  :: ext_dips1,&
                    & ext_dips2
               integer,                      intent(in)  :: n1, n2
               double precision,             intent(out) :: grid(0:,0:)
               double precision,             intent(out) :: gh_lo, gh_int
               double precision,             intent(out) :: gv_lo, gv_int
         end subroutine onm_onm_grid
      end interface
      

      !============================================================
      ! Routines and utilities related to i/o
      interface
         logical function interval_elapsed(interval)
               implicit none
               integer, intent(in)  :: interval
         end function interval_elapsed
      end interface

      interface
         integer function int_val_arg(argk, default_val)
               implicit none
               integer, intent(in)           :: argk
               integer, intent(in), optional :: default_val
         end function int_val_arg
      end interface

      interface
         subroutine read_params_fdat(prnt_size, maxy, n_prev_events,&
              & n_new_events, dat_append)
               implicit none
               !------ The various parameters ---------------------
               double precision,  intent(out) :: prnt_size, maxy
               integer,           intent(out) :: n_prev_events
               integer, optional, intent(out) :: n_new_events
               logical, optional, intent(in)  :: dat_append ! defaults to
               ! .false.
         end subroutine read_params_fdat
      end interface

      interface
         subroutine write_params_fdat(onm_size, maxy, n_events, &
              & param_dev, dat_dev, rwnd_dat)
               implicit none
               !-- arguments ---------------------------------------
               integer,           intent(in)  :: n_events
               double precision,  intent(in)  :: onm_size, maxy
               integer, optional, intent(in)  :: param_dev, dat_dev
               logical, optional, intent(in)  :: rwnd_dat
         end subroutine write_params_fdat
      end interface
      
      
      interface
         subroutine read_params_ufdt(prnt_size, maxy, n_prev_events,&
              & n_new_events)
               implicit none
               double precision,  intent(out) :: prnt_size, maxy
               integer,           intent(out) :: n_prev_events
               integer, optional, intent(out) :: n_new_events
         end subroutine read_params_ufdt
      end interface
      
      interface
         subroutine read_params_old_ufdt(prnt_size, maxy, n_events)
               implicit none
               integer,          intent(out) :: n_events
               double precision, intent(out) :: prnt_size, maxy
         end subroutine read_params_old_ufdt
      end interface

      interface
         subroutine write_params_ufdt(prnt_size, maxy, n_events, param_arg&
              & , ufdt_arg)
               implicit none
               integer,           intent(in)  :: n_events
               double precision,  intent(in)  :: prnt_size, maxy
               integer, optional, intent(in)  :: param_arg, ufdt_arg
         end subroutine write_params_ufdt
      end interface


      interface
         subroutine lcl_write_2d(idev, array)
               implicit none
               integer, intent(in) :: idev
               real,    intent(in) :: array(1:, 1:)
         end subroutine lcl_write_2d
      end interface
      
      interface
         subroutine lcl_read_2d(idev, array)
               implicit none
               integer, intent(in)  :: idev
               real,    intent(out) :: array(1:, 1:)
         end subroutine lcl_read_2d
      end interface

      interface 
         subroutine get_io_devs(iprm_cp, idat_cp, iufdt_cp)
               implicit none
               integer, intent(out) :: iprm_cp, idat_cp, iufdt_cp
         end subroutine get_io_devs
      end interface
      
end module sub_defs

