!**********************************************************************
!     Routines to initiate and terminate the input / output, opening
!     the relevant files etc...
!
!     unit 10:   input/output final parameters
!     unit 12:   input/output final data (formatted)
!     unit 14:   input/output final data (unformatted)
!
! openfiles_fdat    formatted read/write of prms and data
! openoldfiles      formatted read of prms and data
! openfiles_ufdt    (un)formatted read/write of prms (data)
! openoldfiles_ufdt (un)formatted read of prms (data)
!
! Backups, when desired should be performed from the driver routine by
! calling backup_files
!
! OBSOLETE:
! openfiles         formatted read/write of prms and data + do backup
! (kept for compatability)
!
! GPS 30/11/94 -- 7/11/95
!**********************************************************************
subroutine openfiles
      use iomodule
      implicit none
      character*79 string
      integer      i, j, lcl_iargc
      
      i = lcl_iargc()
      if (i < 1) then
         call lcl_getarg(0,string)
         write(0,*) ' Usage: ',string(1:index(string,' ')),'file_basename &
             &[number_of_events]'
         stop
      endif

      call lcl_getarg(1,string)
      j = index(string,' ') - 1

      ! GPS 28/7/95: if the original files exist, then make backup copies. 
      call lcl_system(backup_cmd//string(1:j))

      ! Then open only .dat and .prm for i/o
      open(unit = iprm, file=string(1:j)//'.prm', status = 'unknown')
      open(unit = idat, file=string(1:j)//'.dat', status = 'unknown')

      !------ Obsolete ---------
      ! open(unit = iprm_bak, file=string(1:j)//'.prm.bak', status = 'unknown')
      ! open(unit = idat_bak, file=string(1:j)//'.dat.bak', status = 'unknown')

end subroutine openfiles


! Routine to use when opening files for the purpose of some analysis of
! the data they contain -- i.e. there's no need for any backup files, and
! the files must exist already.
subroutine openoldfiles
      use iomodule
      implicit none
      character*79 string
      integer      i, j, lcl_iargc
      
      i = lcl_iargc()
      if (i < 1) then
         call lcl_getarg(0,string)
         write(0,*) ' Usage: ',string(1:index(string,' ')),'file_basename &
             &[number_of_events]'
         stop
      endif

      call lcl_getarg(1,string)
      j = index(string,' ') - 1
      open(unit = iprm, file=string(1:j)//'.prm', status = 'old')
      open(unit = idat, file=string(1:j)//'.dat', status = 'old')

end subroutine openoldfiles

! Identical to openfiles, but doesn't do backups
subroutine openfiles_fdat
      use iomodule
      implicit none
      character*79 string
      integer      i, j, lcl_iargc
      
      i = lcl_iargc()
      if (i < 1) then
         call lcl_getarg(0,string)
         write(0,*) ' Usage: ',string(1:index(string,' ')),'file_basename &
             &[number_of_events]'
         stop
      endif

      call lcl_getarg(1,string)
      j = index(string,' ') - 1

      ! Then open only .dat and .prm for i/o
      open(unit = iprm, file=string(1:j)//'.prm', status = 'unknown',&
           & position = 'rewind')
      ! must remember to rewind if start_seqnc, or about to read in data.
      open(unit = idat, file=string(1:j)//'.dat', status = 'unknown',&
           & position = 'append')

      !------ Obsolete ---------
      ! open(unit = iprm_bak, file=string(1:j)//'.prm.bak', status = 'unknown')
      ! open(unit = idat_bak, file=string(1:j)//'.dat.bak', status = 'unknown')

end subroutine openfiles_fdat


!**********************************************************************
! Open up files for output. Paramters use formatted output and data uses
! unformatted output. Backups of the files are made using a shell script,
! whose name is found inthe variable backup_cmp
!
! GPS 12/8/95 (CCN7 29
!**********************************************************************
      subroutine openfiles_ufdt
      use iomodule
      implicit none
      character*79 string
      integer      i, j, lcl_iargc
      
      i = lcl_iargc()
      if (i < 1) then
         call lcl_getarg(0,string)
         write(0,*) ' Usage: ',string(1:index(string,' ')),'file_basename &
             &[number_of_events]'
         stop
      endif

      call lcl_getarg(1,string)
      j = index(string,' ') - 1

      ! Then open only .dat and .prm for i/o
      open(unit = iprm, file=string(1:j)//'.prm', status = 'unknown')
      open(unit = iufdt, file=string(1:j)//'.ufdt', form="UNFORMATTED", &
           &status = 'unknown')

end subroutine openfiles_ufdt


! Routine to use when opening files for the purpose of some analysis of
! the data they contain -- i.e. there's no need for any backup files, and
! the files must exist already.
subroutine openoldfiles_ufdt
      use iomodule
      implicit none
      character*79 string
      integer      i, j, lcl_iargc
      
      i = lcl_iargc()
      if (i < 1) then
         call lcl_getarg(0,string)
         write(0,*) ' Usage: ',string(1:index(string,' ')),'file_basename &
             &[number_of_events]'
         stop
      endif

      call lcl_getarg(1,string)
      j = index(string,' ') - 1
      open(unit = iprm, file=string(1:j)//'.prm', status = 'old')
      open(unit = iufdt, file=string(1:j)//'.ufdt', form="UNFORMATTED",&
           & status = 'old')
end subroutine openoldfiles_ufdt


!----------------------------------------------------------------------
! make backup copies of files -- to be called from friver routines. 
!
! GPS 4/11/95 (CCN8 5)
!----------------------------------------------------------------------
subroutine backup_files 
      use iomodule
      implicit none
      character*79 string
      integer      i
      !, lcl_iargc
 
      !-- the heck that there is at least one argument should have been
      ! made elsewhere
      call lcl_getarg(1,string)
      i = index(string,' ') - 1

      ! GPS 28/7/95: if the original files exist, then make backup copies. 
      call lcl_system(backup_cmd//string(1:i))
end subroutine backup_files

!----------------------------------------------------------------------
! Returns true if more than interval seconds have passed since the last
! time the routine returned true. Otherwise, it returns false.
!
! GPS 7/11/95 (CCN8 23)
!----------------------------------------------------------------------
logical function interval_elapsed(interval)
      implicit none
      integer, intent(in)  :: interval
      !------------------------------------------------------------
      integer  :: time(8), seconds, old_seconds = 0

      if (interval < 1 .or. interval > 1800) then
         write(0,*) 'Routine: interval_elapsed'
         write(0,*) 'interval should be between 1 and 1800 seconds'
         stop
      end if
      
      call date_and_time(values=time)           ! get the full date and time
      seconds = 60*time(6) + time(7)

      if ((seconds - old_seconds) >= interval &
          .and. seconds < (old_seconds + 3600 - interval) ) then  ! hour change
         interval_elapsed = .true.

	 ! deal with hour change
         if (seconds + interval > 3559) seconds = seconds - 3600
         old_seconds = seconds
      else
         interval_elapsed = .false.   
      endif
end function interval_elapsed

!----------------------------------------------------------------------
! Return the integer value corresponding to the argk^th command line
! argument. If # args < argk then use default if it is present, otherwise
! output an error message and stop
!
! GPS 8/11/95 (CCN8 23)
!----------------------------------------------------------------------
integer function int_val_arg(argk, default_val)
      implicit none
      integer, intent(in)           :: argk
      integer, intent(in), optional :: default_val
      !------------------------------------------------------------
      integer             :: lcl_iargc, value, i
      character*80 string

      i = lcl_iargc()
      if (i < argk) then
         if (present(default_val)) then
            int_val_arg = default_val
            return
         else
            write(0,*) 'Numerical value of arg',argk,' has been requested'
            write(0,*) 'but only',i,' args present, and no default value&
                 & was provided'
            stop
         end if
      end if

      call lcl_getarg(argk, string)
      int_val_arg = value(string)
end function int_val_arg

      
!**********************************************************************
! Routine to convert string into an integer
!
! GPS 30/11/94
!**********************************************************************

integer function value(string)
      implicit none
      character(*),  intent(in )  ::   string
      character(10), parameter    ::  template = '0123456789'
      character      current
      integer        i, j
      logical        number_started

      value          = 0
      number_started = .false.


loop: do i = 1, len(string)
         current = string(i:i)
         j = index(template, current)
         if (j /= 0) number_started = .true.
         if (number_started) then
            if (j == 0) exit
            value = value * 10 + (j-1)
         endif
      end do loop
end function value


!----------------------------------------------------------------------
! It is sometimes safer to get hold of the  i/o device numbers without
! having direct access to the iomodule module (in case some of args come
! from there).
!
! GPS 20/11/95 (CCN8 35)
!----------------------------------------------------------------------
subroutine get_io_devs(iprm_cp, idat_cp, iufdt_cp)
      use iomodule
      implicit none
      integer, intent(out) :: iprm_cp, idat_cp, iufdt_cp
      !------------------------------------------------------------

      iprm_cp  = iprm
      idat_cp  = idat
      iufdt_cp = iufdt
end subroutine get_io_devs

