!**********************************************************************
!     Module that defines a data type to contain all info relating to
!     data that needs to be i/o-d
! GPS 11/12/94 (CCN3 113)
!**********************************************************************
module data_defs
      !-- see README for details of these vars
      type data_prms
	 double precision  :: y_wvfn, y_int
	 integer           :: n_y           
      end type data_prms

      type(data_prms) data_io
end module data_defs
   

!***************************************************************************   
! Using the full MC distribution of the amplitudes at each r, determine
! the total unitarised and k-pomeron amplitudes 
!
! Eventually put in some kind of safety estimates?
!
! GPS 19/5/95 (CCN5 81)
!****************************************************************************
program profile
      use emergency             !DEBUG
      use gluons                ! Definitions for gluons and dipoles 
      use cuts                  ! The cutoffs being used 
      use sub_defs              ! Some interface blocks 
      use iomodule              ! Open files and get command line args
      use data_defs             ! Def for data type containing data i/o info
      use binning               ! All the parameters for the r and f bins
      use bin_interface         ! Interface blocks for calling those routines
      use fbins_mdl     ! module containg the binned data
      implicit none 

      !---- evolution details ----------------------------------------
      integer                n_events
      double precision       prnt_size, maxy, y
      !---- 'internal' vars ------------------------------------------
      integer                i, ii, i_y
      !---- parameters to do with treatment of data -----------------
      integer, parameter    :: npom = 8
      double precision      :: amp(0:npom), rad_mn
      !---- To save repeated evaluation of parameters of bins -------
      double precision,  allocatable, dimension(:)     :: ring_areas
      double precision,  allocatable, dimension(:,:)   :: fvals
      !------------------------------------------------------------------------

      !----- start everything the proper way --------------------
      call init_all

      ! Figure out which of the y values to use
      i_y = int_val_arg(2, default_val = 1)

      y = 2d0*(maxy + (i_y - data_io%n_y) * data_io%y_int)
      write(0,*) 'The value of y is ', y
         
      amp = 0d0
      do i = 0, bn%nr
         rad_mn = (rad_lo(i) + rad_lo(i+1))/2d0
         do ii = 0, npom
            amp(ii) = sum(fbins(:,i,i_y) * fvals(:,ii))
         end do

         write(6,'(25es15.4)') rad_mn, amp
      end do
      
      !----- finish cleanly --------------------------
      deallocate(fbins)
                                                                        
!---------------------------------------------------------------------- 
                                              
contains 
      

!----------------------------------------------------------------------
! Store things like radii of rings corresponding to bins in r and values
! for k-pomeron amplitudes associated with bins in f
!
! GPS 12/8/95 (CCN 7 31)
!----------------------------------------------------------------------
subroutine bin_shrtcts
      use constants
      use binning
      use bin_interface
      implicit none
      integer i, j
      double precision  f

      allocate (ring_areas(0:bn%nr))
      do i = 0, bn%nr
         ring_areas(i) = pi * (rad_lo(i+1)**2 - rad_lo(i)**2)
      end do

      allocate (fvals(-1:bn%nf, 0:npom))
      do i = -1, bn%nf
         f = f_of_bin(i)
         fvals(i,0) = 1d0 - exp(-f)
         fvals(i,1) = f
         do j = 2, npom
            fvals(i,j) = fvals(i,j-1) * f / j
         end do
      end do
end subroutine bin_shrtcts


!----------------------------------------------------------------------
! This tests to see if one has reached the boundaries of the binning in
! either r or f
!
! GPS 12/8/95 (CCN7 33)
!----------------------------------------------------------------------
subroutine test_ovflw
      use binning 
      use data_defs
      implicit none
      integer i_y
      double precision  y, r_ovflw, f_ovflw

      do i_y = 1, data_io%n_y
         y = 2d0*(maxy + (i_y - data_io%n_y) * data_io%y_int)
         
         ! Check first for r
         r_ovflw = sum(fbins(0:bn%nf, bn%nr, i_y))
         f_ovflw = sum(fbins(bn%nf, 0:bn%nr, i_y))
         if (r_ovflw /= 0d0 .or. f_ovflw /= 0d0) then
            write(0,*) "At y =",y," There was overflow in the binning:"
            write(0,*) "r:", r_ovflw," events"
            write(0,*) "f:", f_ovflw," events x bins"
         end if
      end do
end subroutine test_ovflw



!----------------------------------------------------------------------
! Keep all common initialisation out of the way
!----------------------------------------------------------------------
subroutine init_all
      use sub_defs
      use bin_interface
      implicit none
      integer i

      call read_params_old_ufdt(prnt_size, maxy, n_events)
      call allc_read_fbins(data_io%n_y)

      call test_ovflw
      call bin_shrtcts
end subroutine init_all

end program profile

